------------------------------------------------------------------
------------
--                         Copyright (C) 2006-200?
--                     Universidad de Cantabria, SPAIN
--                        http://www.ctr.unican.es/

with CORBA;
with CORBA.ORB;
with CORBA.Object;
with CORBA.Impl;
with CORBA.Policy;

with PortableServer.POA;
with PortableServer.POA.Helper;
with PortableServer.POAManager;

--  RTCorba
with RTCORBA.RTORB;
with RTCORBA.RTORB.Helper;
with RTCORBA.PriorityMapping.Linear;
with RTCORBA.PriorityModelPolicy;
with RTCORBA.ThreadpoolPolicy;

with RTPortableServer.POA;
with RTPortableServer.POA.Helper;
with PolyORB.RTCORBA_P.Setup;
--  End of RTCorba

with Kernel_Console; use Kernel_Console;

package body Inicio is

   use CORBA.ORB;
   use CORBA.Policy.IDL_SEQUENCE_Policy;
   use RTCORBA;
   use RTCORBA.RTORB;

   procedure Servidor
     (Object   : in CORBA.Impl.Object_Ptr;
      Root_POA : out PortableServer.POA.Local_Ref;
      IOR      : out CORBA.Object.Ref)
   is
      Argv  : CORBA.ORB.Arg_List := CORBA.ORB.Command_Line_Arguments;
      idorb : CORBA.ORB.ORBid;

   begin
      --  Root POA
      Root_POA :=
         PortableServer.POA.Helper.To_Local_Ref
           (CORBA.ORB.Resolve_Initial_References
               (CORBA.ORB.To_CORBA_String ("RootPOA")));

      --  Activate POA Manager (Holding at the beginning)
      PortableServer.POAManager.Activate
        (PortableServer.POA.Get_The_POAManager (Root_POA));

      --  Servant - Object binding (IOR)
      IOR :=
         PortableServer.POA.Servant_To_Reference
           (Root_POA,
            PortableServer.Servant (Object));
   end Servidor;

   function Create_Child_RT_POA
     (Name           : String;
      Static_Threads : Natural;
      Priority       : RTCORBA.Priority)
      return           RTPortableServer.POA.Local_Ref;
   --  Create a RT POA with the following configuration:
   --  "Static_Threads" threads, at priority "Priority".

   -------------------------
   -- Create_Child_RT_POA --
   -------------------------

   function Create_Child_RT_POA
     (Name           : String;
      Static_Threads : Natural;
      Priority       : RTCORBA.Priority)
      return           RTPortableServer.POA.Local_Ref
   is
      RT_ORB   : RTCORBA.RTORB.Local_Ref;
      Root_POA : PortableServer.POA.Local_Ref;

      Priority_Model_Policy_Ref : RTCORBA.PriorityModelPolicy.Local_Ref;
      Thread_Pool_Id            : RTCORBA.ThreadpoolId;
      Thread_Pool_Policy_Ref    : RTCORBA.ThreadpoolPolicy.Local_Ref;
      Policies                  : CORBA.Policy.PolicyList;
      Result                    : RTPortableServer.POA.Local_Ref;

      --  Configure priorities
      Base_Priority      : constant RTCORBA.Priority := 0;
      Default_Priority_1 : constant RTCORBA.Priority := 8_000;
      Default_Priority_2 : constant RTCORBA.Priority := 25_000;

   begin

      --  Retrieve Root POA
      Root_POA :=
         PortableServer.POA.Helper.To_Local_Ref
           (CORBA.ORB.Resolve_Initial_References
               (CORBA.ORB.To_CORBA_String ("RootPOA")));

      --  Retrieve RT ORB
      RT_ORB :=
         RTCORBA.RTORB.Helper.To_Local_Ref
           (Resolve_Initial_References (To_CORBA_String ("RTORB")));

      --  Client_Propagated
      Priority_Model_Policy_Ref :=
         Create_Priority_Model_Policy
           (RT_ORB,
            CLIENT_PROPAGATED,
            Default_Priority_2);--  Default Priority Non-RT

      --  Create Threadpool
      Thread_Pool_Id :=
         RTCORBA.RTORB.Create_Threadpool
           (RT_ORB,
            Stacksize               => 0,
            Static_Threads          => CORBA.Unsigned_Long (Static_Threads),
            Dynamic_Threads         => 0,
            Default_Priority        => Priority,
            Allow_Request_Buffering => False,
            Max_Buffered_Requests   => 0,
            Max_Request_Buffer_Size => 0);--  0 is the only value accepted

      --  Construct Thread Pool policy from previous threadpool
      Thread_Pool_Policy_Ref :=
         RTCORBA.RTORB.Create_Threadpool_Policy (RT_ORB, Thread_Pool_Id);

      --  Add policies to list (CORBA.Policy.IDL_Sequence_Policy)
      Append (Policies, CORBA.Policy.Ref (Priority_Model_Policy_Ref));

      Append (Policies, CORBA.Policy.Ref (Thread_Pool_Policy_Ref));

      Result :=
         RTPortableServer.POA.Helper.To_Local_Ref
           (PortableServer.POA.Create_POA
               (Root_POA,
                CORBA.To_CORBA_String (Name),
                PortableServer.POA.Get_The_POAManager (Root_POA),
                Policies));

      return Result;
   end Create_Child_RT_POA;

   --  Create a RTPOA with lanes
   function Create_Child_RT_POA_Lanes
     (Name : String)
      return RTPortableServer.POA.Local_Ref
   is
      RT_ORB   : RTCORBA.RTORB.Local_Ref;
      Root_POA : PortableServer.POA.Local_Ref;

      Priority_Model_Policy_Ref : RTCORBA.PriorityModelPolicy.Local_Ref;
      Thread_Pool_Id            : RTCORBA.ThreadpoolId;
      Thread_Pool_Policy_Ref    : RTCORBA.ThreadpoolPolicy.Local_Ref;
      Policies                  : CORBA.Policy.PolicyList;
      Result                    : RTPortableServer.POA.Local_Ref;

      --  Lanes Variables
      Lane_1 : RTCORBA.ThreadpoolLane;
      Lane_2 : RTCORBA.ThreadpoolLane;
      Lane_3 : RTCORBA.ThreadpoolLane;
      Lane_4 : RTCORBA.ThreadpoolLane;
      Lane_5 : RTCORBA.ThreadpoolLane;

      Lanes : RTCORBA.ThreadpoolLanes;

      --  Priorities
      Base_Priority      : constant RTCORBA.Priority := 0;
      Default_Priority_1 : constant RTCORBA.Priority := 8_000;
      Default_Priority_2 : constant RTCORBA.Priority := 25_000;

   begin
      --  Retrieve Root POA
      Root_POA :=
         PortableServer.POA.Helper.To_Local_Ref
           (CORBA.ORB.Resolve_Initial_References
               (CORBA.ORB.To_CORBA_String ("RootPOA")));

      --  Retrieve RT ORB
      RT_ORB :=
         RTCORBA.RTORB.Helper.To_Local_Ref
           (Resolve_Initial_References (To_CORBA_String ("RTORB")));

      -- *** Client_Propagated ***
      Priority_Model_Policy_Ref :=
         Create_Priority_Model_Policy
           (RT_ORB,
            CLIENT_PROPAGATED,
            Base_Priority);--  Default Priority Non-RT

      --  Configure priorities
      --  Lanes Parameters
      Lane_1.Lane_Priority   := 25_000;
      Lane_1.Static_Threads  := 1;
      Lane_1.Dynamic_Threads := 0;

      Lane_2.Lane_Priority   := 2_000;
      Lane_2.Static_Threads  := 1;
      Lane_2.Dynamic_Threads := 0;

      --
      --              --  Lanes Parameters
      Lane_3.Lane_Priority   := 8_000;
      Lane_3.Static_Threads  := 1;
      Lane_3.Dynamic_Threads := 0;
      --
      Lane_4.Lane_Priority   := 10_000;
      Lane_4.Static_Threads  := 1;
      Lane_4.Dynamic_Threads := 0;
      --
      --              --  Lanes Parameters
      Lane_5.Lane_Priority   := 12_000;
      Lane_5.Static_Threads  := 1;
      Lane_5.Dynamic_Threads := 0;
      --

      --  Create Lanes
      Append (Lanes, Lane_1);
      Append (Lanes, Lane_2);
      Append (Lanes, Lane_3);
      Append (Lanes, Lane_4);
      Append (Lanes, Lane_5);

      --  Create Threadpool

      Thread_Pool_Id :=
         RTCORBA.RTORB.Create_Threadpool_With_Lanes
           (RT_ORB,
            Stacksize               => 262_164,
            Lanes                   => Lanes,
            Allow_Borrowing         => False,
            Allow_Request_Buffering => True,
            Max_Buffered_Requests   => 1,
            Max_Request_Buffer_Size => 0);

      --  Construct Thread Pool policy from previous threadpool

      Thread_Pool_Policy_Ref :=
         RTCORBA.RTORB.Create_Threadpool_Policy (RT_ORB, Thread_Pool_Id);

      Append (Policies, CORBA.Policy.Ref (Priority_Model_Policy_Ref));

      Append (Policies, CORBA.Policy.Ref (Thread_Pool_Policy_Ref));

      Result :=
         RTPortableServer.POA.Helper.To_Local_Ref
           (PortableServer.POA.Create_POA
               (Root_POA,
                CORBA.To_CORBA_String (Name),
                PortableServer.POA.Get_The_POAManager (Root_POA),
                Policies));

      return Result;
   end Create_Child_RT_POA_Lanes;

   procedure Servidor_RTCorba
     (Object : in out CORBA.Impl.Object_Ptr;
      RT_POA : out RTPortableServer.POA.Local_Ref;
      IOR    : out CORBA.Object.Ref)
   is

      Priority_Mapping : RTCORBA.PriorityMapping.Linear.Object;
      idorb            : CORBA.ORB.ORBid    :=
         CORBA.ORB.To_CORBA_String ("ORB");
      arg              : CORBA.ORB.Arg_List :=
         CORBA.ORB.Command_Line_Arguments;
      Root_POA         : PortableServer.POA.Local_Ref;

   begin
      PolyORB.RTCORBA_P.Setup.Set_Priority_Mapping (Priority_Mapping);

      --  Retrieve Root POA
      Root_POA :=
         PortableServer.POA.Helper.To_Local_Ref
           (CORBA.ORB.Resolve_Initial_References
               (CORBA.ORB.To_CORBA_String ("RootPOA")));

      PortableServer.POAManager.Activate
        (PortableServer.POA.Get_The_POAManager (Root_POA));

      --  Select if lanes are to be used commenting/uncommenting those lines

      --  No lanes option
      --        RT_POA := Create_Child_RT_POA(Name => "Echo_RTPOA",
      --                                      Static_Threads => 5,
      --                                      Priority => 25_000);
      --  Lanes option
      RT_POA := Create_Child_RT_POA_Lanes (Name => "Echo_RTPOA");

   end Servidor_RTCorba;

end Inicio;
