------------------------------------------------------
------------------------
--                         Copyright (C) 2006-200?
--                     Universidad de Cantabria, SPAIN
--                        http://www.ctr.unican.es/

--  RTCorba example: Sum
--  Node: Remote
--  Protocol: RATO-RTEP
--  OS: Linux
--  Priority: Set at configuration time
--  Policies: RATO and Thread_Per_Target
--  Scheduling: Priorities

with MaRTE_OS;

--  CORBA
with CORBA.Impl;
with CORBA.Object;
with CORBA.ORB;

with CORBA.Policy;
with CORBA;
with CORBA.Object.Policies;

with PortableServer.POA;
with RTPortableServer.POA;

with Suma.Impl; use Suma.Impl;

--  System Configuration
with Server_Configuration;
pragma Elaborate_All (Server_Configuration);

with Inicio;

--  Debug and exceptions handling
with Ada.Exceptions;
with Kernel_Console; use Kernel_Console;

--  RTEP
with PolyORB.RTEP_MAC;
with PolyORB.API.RTEP_MAC.Scheduling.Priorities;
with RTEP;                                       use RTEP;

procedure Server is

   --  Short names
   package Sched renames PolyORB.API.RTEP_MAC.Scheduling.Priorities;
   package RTEP_MAC renames PolyORB.RTEP_MAC;

   --  Main task priority: It must be less than created task to serve requests
   pragma Priority (1);

   --  Variables
   IOR_ref : CORBA.Object.Ref; --  IOR to publish object location
   RT_POA  : RTPortableServer.POA.Local_Ref;
   --  POA		: PortableServer.POA.Ref;
   Object      : CORBA.Impl.Object_Ptr;
   Policy_List : CORBA.Policy.PolicyList;
   ResultP     : CORBA.Boolean;

   --  Command line arguments
   Argv  : CORBA.ORB.Arg_List := CORBA.ORB.Command_Line_Arguments;
   idorb : CORBA.ORB.ORBid;

   --  Endpoints to be created
   Number_Of_Endpoints : Natural := 5;

begin

   Put ("Running PolyORB & CORBA over MaRTE OS (not debug)");

   --  Debug facilities
   PolyORB.API.RTEP_MAC.Enable_Debug (False);

   --  ORB Init
   Put ("Init ... ");
   idorb := CORBA.ORB.To_CORBA_String ("ORB");
   CORBA.ORB.Init (idorb, Argv);

   Put ("RTPOA ... ");
   --  Create remote object
   Object := new Suma.Impl.Object;

   --  RTPOA init
   Inicio.Servidor_RTCorba
     (Object => Object,
      RT_POA => RT_POA,
      IOR    => IOR_ref);

   --  POA init
   --     Inicio.Servidor(Object => Object,
   --                    Root_POA => POA,
   --                    IOR => IOR_ref);



   --  Explicit binding
   --  CORBA.Object.Policies.Validate_Connection(IOR_Ref, Policy_List,
   --ResultP);

   Put ("RT environment ... ");

   --  ******************** RTEP ***************************

   declare

      P            : Sched.Priorities_Message_Scheduling_Parameters;
      P_Ref        : Sched.Priorities_Message_Scheduling_Parameters_Ref;
      Endpoint_Out : RTEP_MAC.Endpoint_Id;
      Node_To_Reply : Station_ID := 2;

      J : Natural := 2;  --  Loop variable

   begin

      while J < (Number_Of_Endpoints + 2) loop

         --  Scheduling API
         P.P_in := RTEP.Priority (30 + J);
         Sched.Set_Scheduling_Params (P, P_Ref);

         --  Setting receive endpoints
         Sched.Create_Receive_Endpoint
           (Params    => P_Ref,
            Dest_Node => Node_To_Reply,
            Event_ID  => J,
            Channel   => RTEP_MAC.Channel (J),
            Endpoint  => Endpoint_Out);

         J := J + 1;
      end loop;

   end;

   --  We need to rewrite the IOR because AP is not created at init phase
   IOR_ref :=
      PortableServer.POA.Servant_To_Reference
        (PortableServer.POA.Local_Ref (RT_POA),
         PortableServer.Servant (Object));

   Put (" Launching! ");

   --  Launch the server
   CORBA.ORB.Run;
   Put (" End of the ORB world ");

exception
   when E : others =>
      Put ("Received exception: ");
      Put (Ada.Exceptions.Exception_Information (E));

end Server;
