------------------------------------------------------
-------------
-------------
--                         Copyright (C) 2006-200?
--                     Universidad de Cantabria, SPAIN
--                        http://www.ctr.unican.es/

--  RTCorba example: Sum
--  Node: Local
--  Protocol: RTC-RTEP
--  OS: Linux Fedora Core 4
--  RTCorba Priority: Client Propagated
--  Policies: Workers and Thread_Pool

with MaRTE_OS;

--  Remote Object
with Suma;

--  Corba
with CORBA;
with CORBA.ORB;
with CORBA.Object;
with CORBA.Impl;

--  RTCorba
with RTCORBA;
with RTCORBA.Current;
with RTCORBA.Current.Helper;
with RTCORBA.PriorityMapping.Linear;
with PolyORB.RTCORBA_P.Setup;

--  System Configuration
with Client_Configuration;

--  RTEP
with RTEP_IOR;
with PolyORB.API.RTEP_MAC.RTC;

--  RTEP QoS (Service context priorities)
with PolyORB.QoS.RTEP_Parameters; use PolyORB.QoS.RTEP_Parameters;

with RTEP; use RTEP; --  Priority

--  Output facilities
with PolyORB.Tasking.Threads; use PolyORB.Tasking.Threads;
with Ada.Dynamic_Priorities;  use Ada.Dynamic_Priorities;
with System;
with Ada.Exceptions;
with Kernel_Console; use Kernel_Console;

procedure Client is

   pragma Priority (8);

   Principal          : Boolean := True;
   Number_Of_Measures : Natural := 20;

   Ior_Ref : Suma.Ref;

   --  Remote request params
   --  Current thread managing
   ThreadCurrent : RTCORBA.Current.Local_Ref;

   --  Priority mapping CORBA-ADA
   Priority_Mapping : RTCORBA.PriorityMapping.Linear.Object;

   --  Priority table
   Client_Priority_1 : array (1 .. 5) of RTCORBA.Priority :=
     (25_000,
      2_000,
      8_000,
      10_000,
      12_000);

   Break_Time : Boolean := True; --  Time between requests

   Argv  : CORBA.ORB.Arg_List := CORBA.ORB.Command_Line_Arguments;
   idorb : CORBA.ORB.ORBid;

   --  Tasks to make requests: Each worker configures its parameters and make
   --  request to remote object SUM. The operation is Id + 2;
   task type Worker (Id : Natural) is
      entry Empieza;
      entry Prioridad;
   end Worker;

   task body Worker is
      --  Local variables
      Res  : CORBA.Long;
      Num1 : CORBA.Long;
      Num2 : CORBA.Long := 2;

   begin

      accept Prioridad;
      --  Priorities
      ThreadCurrent :=
         RTCORBA.Current.Helper.To_Local_Ref
           (CORBA.ORB.Resolve_Initial_References
               (CORBA.ORB.To_CORBA_String ("RTCurrent")));

      RTCORBA.Current.Set_The_Priority
        (ThreadCurrent,
         Client_Priority_1 (Id));

      --  Recover IOR from file

      Num1 := CORBA.Long (Id);

      --  ******************** RTEP ***************************
      declare
         Test_Node : PolyORB.QoS.RTEP_Parameters.Node_Role := CLIENT;
         Current_Thread_Priority : RTEP.Priority;

      begin

         Current_Thread_Priority :=
            RTEP.Priority (Ada.Dynamic_Priorities.Get_Priority);

         Put
           (" Thread: " &
            Image (Current_Task) &
            " P->" &
            System.Any_Priority'Image (Integer (Current_Thread_Priority)) &
            " ");

         --  RTEP Paramters Configuration
         PolyORB.API.RTEP_MAC.RTC.Configure_RTEP_Parameters
           (CORBA.To_Standard_String
               (CORBA.Object.Object_To_String (Ior_Ref)),
            P_in     => Current_Thread_Priority - 1,
            O_out    => Current_Thread_Priority,
            P_out    => Current_Thread_Priority + 1,
            Event_ID => Id,
            Node     => Test_Node);
      end;

      --  ******************** RTEP ***************************

      accept Empieza;

      loop
         for I in 1 .. Number_Of_Measures loop
            --  Remote invocation
            Res := Suma.SumaNum (Ior_Ref, Num1, Num2);
            if Break_Time then
               delay (0.01);
            end if;

         end loop;

         Put
           ("Worker: " &
            Natural'Image (Id) &
            " Result: " &
            CORBA.Long'Image (Res));
         New_Line;

      end loop;
   end Worker;

   w1 : Worker (1);
   w2 : Worker (2);
   w3 : Worker (3);
   w4 : Worker (4);
   w5 : Worker (5);

begin
   PolyORB.API.RTEP_MAC.Enable_Debug (False);

   Put ("PolyORB over MaRTE... Starting");
   New_Line;
   idorb := CORBA.ORB.To_CORBA_String ("ORB");

   --  Init ORB
   CORBA.ORB.Init (idorb, Argv);

   Put ("Init ... Ok");

   --  Establish a Priority Mapping
   PolyORB.RTCORBA_P.Setup.Set_Priority_Mapping (Priority_Mapping);

   --  Recover IOR from file
   CORBA.ORB.String_To_Object
     (CORBA.To_CORBA_String (RTEP_IOR.IOR_Suma),
      Ior_Ref);

   Put ("Setting up RT enviroment...");
   New_Line;

   delay (5.0);

   w1.Prioridad;
   delay (0.1);
   w2.Prioridad;
   delay (0.1);
   w3.Prioridad;
   delay (0.1);
   w4.Prioridad;
   delay (0.1);
   w5.Prioridad;
   delay (0.1);

   w1.Empieza;
   w2.Empieza;
   w3.Empieza;
   w4.Empieza;
   w5.Empieza;

exception
   when E : others =>
      Put ("Received exception: ");
      Put (Ada.Exceptions.Exception_Information (E));

end Client;
